@@ -171,6 +171,7 @@ var LibraryPThreadStub = {
   pthread_rwlock_wrlock: function() { return 0; },
   pthread_rwlock_trywrlock: function() { return 0; },
   pthread_rwlock_timedwrlock: function() { return 0; },
+  pthread_rwlock_unlock: function() { return 0; },
 
   pthread_rwlockattr_init: function() { return 0; },
   pthread_rwlockattr_destroy: function() { return 0; },
@@ -261,11 +262,11 @@ var LibraryPThreadStub = {
     }
   },
 
-  __atomic_fetch_add_8__deps: ['llvm_uadd_with_overflow_i64'],
+  __atomic_fetch_add_8__deps: ['i64Add'],
   __atomic_fetch_add_8: function(ptr, vall, valh, memmodel) {
     var l = {{{ makeGetValue('ptr', 0, 'i32') }}};
     var h = {{{ makeGetValue('ptr', 4, 'i32') }}};
-    {{{ makeSetValue('ptr', 0, '_llvm_uadd_with_overflow_i64(l, h, vall, valh)', 'i32') }}};
+    {{{ makeSetValue('ptr', 0, '_i64Add(l, h, vall, valh)', 'i32') }}};
     {{{ makeSetValue('ptr', 4, 'Runtime["getTempRet0"]()', 'i32') }}};
     {{{ makeStructuralReturn(['l', 'h']) }}};
   },
